"""
Blessed Asana connector for Airbyte SDK.

Auto-generated from OpenAPI specification.
"""

from .connector import AsanaConnector
from .models import (
    AsanaAuthConfig,
    TaskCompactCreatedBy,
    TaskCompact,
    Task,
    TaskResponse,
    TasksListNextPage,
    TasksList,
    ProjectCompact,
    ProjectWorkspace,
    ProjectOwner,
    ProjectCurrentStatusCreatedBy,
    ProjectCurrentStatusAuthor,
    ProjectCurrentStatus,
    ProjectMembersItem,
    ProjectTeam,
    ProjectFollowersItem,
    ProjectCurrentStatusUpdate,
    Project,
    ProjectResponse,
    ProjectsListNextPage,
    ProjectsList,
    WorkspaceCompact,
    Workspace,
    WorkspaceResponse,
    WorkspacesListNextPage,
    WorkspacesList,
    UserCompact,
    UserWorkspacesItem,
    User,
    UserResponse,
    UsersListNextPage,
    UsersList,
    TeamCompact,
    TeamOrganization,
    Team,
    TeamResponse,
    TeamsListNextPage,
    TeamsList,
    AttachmentCompact,
    AttachmentParent,
    Attachment,
    AttachmentResponse,
    AttachmentsListNextPage,
    AttachmentsList,
    TagCompact,
    TagWorkspace,
    Tag,
    TagResponse,
    TagsListNextPage,
    TagsList,
    SectionCompact,
    SectionProject,
    Section,
    SectionResponse,
    SectionsListNextPage,
    SectionsList,
    TasksListResultMeta,
    ProjectTasksListResultMeta,
    WorkspaceTaskSearchListResultMeta,
    ProjectsListResultMeta,
    TaskProjectsListResultMeta,
    TeamProjectsListResultMeta,
    WorkspaceProjectsListResultMeta,
    WorkspacesListResultMeta,
    UsersListResultMeta,
    WorkspaceUsersListResultMeta,
    TeamUsersListResultMeta,
    WorkspaceTeamsListResultMeta,
    UserTeamsListResultMeta,
    AttachmentsListResultMeta,
    WorkspaceTagsListResultMeta,
    ProjectSectionsListResultMeta,
    TaskSubtasksListResultMeta,
    TaskDependenciesListResultMeta,
    TaskDependentsListResultMeta,
    AsanaExecuteResult,
    AsanaExecuteResultWithMeta,
    TasksListResult,
    ProjectTasksListResult,
    TasksGetResult,
    WorkspaceTaskSearchListResult,
    ProjectsListResult,
    ProjectsGetResult,
    TaskProjectsListResult,
    TeamProjectsListResult,
    WorkspaceProjectsListResult,
    WorkspacesListResult,
    WorkspacesGetResult,
    UsersListResult,
    UsersGetResult,
    WorkspaceUsersListResult,
    TeamUsersListResult,
    TeamsGetResult,
    WorkspaceTeamsListResult,
    UserTeamsListResult,
    AttachmentsListResult,
    AttachmentsGetResult,
    WorkspaceTagsListResult,
    TagsGetResult,
    ProjectSectionsListResult,
    SectionsGetResult,
    TaskSubtasksListResult,
    TaskDependenciesListResult,
    TaskDependentsListResult
)
from .types import (
    TasksListParams,
    ProjectTasksListParams,
    TasksGetParams,
    WorkspaceTaskSearchListParams,
    ProjectsListParams,
    ProjectsGetParams,
    TaskProjectsListParams,
    TeamProjectsListParams,
    WorkspaceProjectsListParams,
    WorkspacesListParams,
    WorkspacesGetParams,
    UsersListParams,
    UsersGetParams,
    WorkspaceUsersListParams,
    TeamUsersListParams,
    TeamsGetParams,
    WorkspaceTeamsListParams,
    UserTeamsListParams,
    AttachmentsListParams,
    AttachmentsGetParams,
    AttachmentsDownloadParams,
    WorkspaceTagsListParams,
    TagsGetParams,
    ProjectSectionsListParams,
    SectionsGetParams,
    TaskSubtasksListParams,
    TaskDependenciesListParams,
    TaskDependentsListParams
)

__all__ = ["AsanaConnector", "AsanaAuthConfig", "TaskCompactCreatedBy", "TaskCompact", "Task", "TaskResponse", "TasksListNextPage", "TasksList", "ProjectCompact", "ProjectWorkspace", "ProjectOwner", "ProjectCurrentStatusCreatedBy", "ProjectCurrentStatusAuthor", "ProjectCurrentStatus", "ProjectMembersItem", "ProjectTeam", "ProjectFollowersItem", "ProjectCurrentStatusUpdate", "Project", "ProjectResponse", "ProjectsListNextPage", "ProjectsList", "WorkspaceCompact", "Workspace", "WorkspaceResponse", "WorkspacesListNextPage", "WorkspacesList", "UserCompact", "UserWorkspacesItem", "User", "UserResponse", "UsersListNextPage", "UsersList", "TeamCompact", "TeamOrganization", "Team", "TeamResponse", "TeamsListNextPage", "TeamsList", "AttachmentCompact", "AttachmentParent", "Attachment", "AttachmentResponse", "AttachmentsListNextPage", "AttachmentsList", "TagCompact", "TagWorkspace", "Tag", "TagResponse", "TagsListNextPage", "TagsList", "SectionCompact", "SectionProject", "Section", "SectionResponse", "SectionsListNextPage", "SectionsList", "TasksListResultMeta", "ProjectTasksListResultMeta", "WorkspaceTaskSearchListResultMeta", "ProjectsListResultMeta", "TaskProjectsListResultMeta", "TeamProjectsListResultMeta", "WorkspaceProjectsListResultMeta", "WorkspacesListResultMeta", "UsersListResultMeta", "WorkspaceUsersListResultMeta", "TeamUsersListResultMeta", "WorkspaceTeamsListResultMeta", "UserTeamsListResultMeta", "AttachmentsListResultMeta", "WorkspaceTagsListResultMeta", "ProjectSectionsListResultMeta", "TaskSubtasksListResultMeta", "TaskDependenciesListResultMeta", "TaskDependentsListResultMeta", "AsanaExecuteResult", "AsanaExecuteResultWithMeta", "TasksListResult", "ProjectTasksListResult", "TasksGetResult", "WorkspaceTaskSearchListResult", "ProjectsListResult", "ProjectsGetResult", "TaskProjectsListResult", "TeamProjectsListResult", "WorkspaceProjectsListResult", "WorkspacesListResult", "WorkspacesGetResult", "UsersListResult", "UsersGetResult", "WorkspaceUsersListResult", "TeamUsersListResult", "TeamsGetResult", "WorkspaceTeamsListResult", "UserTeamsListResult", "AttachmentsListResult", "AttachmentsGetResult", "WorkspaceTagsListResult", "TagsGetResult", "ProjectSectionsListResult", "SectionsGetResult", "TaskSubtasksListResult", "TaskDependenciesListResult", "TaskDependentsListResult", "TasksListParams", "ProjectTasksListParams", "TasksGetParams", "WorkspaceTaskSearchListParams", "ProjectsListParams", "ProjectsGetParams", "TaskProjectsListParams", "TeamProjectsListParams", "WorkspaceProjectsListParams", "WorkspacesListParams", "WorkspacesGetParams", "UsersListParams", "UsersGetParams", "WorkspaceUsersListParams", "TeamUsersListParams", "TeamsGetParams", "WorkspaceTeamsListParams", "UserTeamsListParams", "AttachmentsListParams", "AttachmentsGetParams", "AttachmentsDownloadParams", "WorkspaceTagsListParams", "TagsGetParams", "ProjectSectionsListParams", "SectionsGetParams", "TaskSubtasksListParams", "TaskDependenciesListParams", "TaskDependentsListParams"]



"""
asana connector.
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, AsyncIterator, overload
try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from .connector_model import AsanaConnectorModel

from .types import (
    AttachmentsDownloadParams,
    AttachmentsGetParams,
    AttachmentsListParams,
    ProjectSectionsListParams,
    ProjectTasksListParams,
    ProjectsGetParams,
    ProjectsListParams,
    SectionsGetParams,
    TagsGetParams,
    TaskDependenciesListParams,
    TaskDependentsListParams,
    TaskProjectsListParams,
    TaskSubtasksListParams,
    TasksGetParams,
    TasksListParams,
    TeamProjectsListParams,
    TeamUsersListParams,
    TeamsGetParams,
    UserTeamsListParams,
    UsersGetParams,
    UsersListParams,
    WorkspaceProjectsListParams,
    WorkspaceTagsListParams,
    WorkspaceTaskSearchListParams,
    WorkspaceTeamsListParams,
    WorkspaceUsersListParams,
    WorkspacesGetParams,
    WorkspacesListParams,
)

if TYPE_CHECKING:
    from .models import AsanaAuthConfig
# Import specific auth config classes for multi-auth isinstance checks
from .models import AsanaOauth2AuthConfig, AsanaPersonalAccessTokenAuthConfig
# Import response models and envelope models at runtime
from .models import (
    AsanaExecuteResult,
    AsanaExecuteResultWithMeta,
    TasksListResult,
    ProjectTasksListResult,
    TasksGetResult,
    WorkspaceTaskSearchListResult,
    ProjectsListResult,
    ProjectsGetResult,
    TaskProjectsListResult,
    TeamProjectsListResult,
    WorkspaceProjectsListResult,
    WorkspacesListResult,
    WorkspacesGetResult,
    UsersListResult,
    UsersGetResult,
    WorkspaceUsersListResult,
    TeamUsersListResult,
    TeamsGetResult,
    WorkspaceTeamsListResult,
    UserTeamsListResult,
    AttachmentsListResult,
    AttachmentsGetResult,
    WorkspaceTagsListResult,
    TagsGetResult,
    ProjectSectionsListResult,
    SectionsGetResult,
    TaskSubtasksListResult,
    TaskDependenciesListResult,
    TaskDependentsListResult,
)


class AsanaConnector:
    """
    Type-safe Asana API connector.

    Auto-generated from OpenAPI specification with full type safety.
    """

    connector_name = "asana"
    connector_version = "0.1.5"
    vendored_sdk_version = "0.1.0"  # Version of vendored connector-sdk

    # Map of (entity, action) -> has_extractors for envelope wrapping decision
    _EXTRACTOR_MAP = {
        ("tasks", "list"): True,
        ("project_tasks", "list"): True,
        ("tasks", "get"): True,
        ("workspace_task_search", "list"): True,
        ("projects", "list"): True,
        ("projects", "get"): True,
        ("task_projects", "list"): True,
        ("team_projects", "list"): True,
        ("workspace_projects", "list"): True,
        ("workspaces", "list"): True,
        ("workspaces", "get"): True,
        ("users", "list"): True,
        ("users", "get"): True,
        ("workspace_users", "list"): True,
        ("team_users", "list"): True,
        ("teams", "get"): True,
        ("workspace_teams", "list"): True,
        ("user_teams", "list"): True,
        ("attachments", "list"): True,
        ("attachments", "get"): True,
        ("attachments", "download"): False,
        ("workspace_tags", "list"): True,
        ("tags", "get"): True,
        ("project_sections", "list"): True,
        ("sections", "get"): True,
        ("task_subtasks", "list"): True,
        ("task_dependencies", "list"): True,
        ("task_dependents", "list"): True,
    }

    # Map of (entity, action) -> {python_param_name: api_param_name}
    # Used to convert snake_case TypedDict keys to API parameter names in execute()
    _PARAM_MAP = {
        ('tasks', 'list'): {'limit': 'limit', 'offset': 'offset', 'project': 'project', 'workspace': 'workspace', 'section': 'section', 'assignee': 'assignee', 'completed_since': 'completed_since', 'modified_since': 'modified_since'},
        ('project_tasks', 'list'): {'project_gid': 'project_gid', 'limit': 'limit', 'offset': 'offset', 'completed_since': 'completed_since'},
        ('tasks', 'get'): {'task_gid': 'task_gid'},
        ('workspace_task_search', 'list'): {'workspace_gid': 'workspace_gid', 'limit': 'limit', 'offset': 'offset', 'text': 'text', 'completed': 'completed', 'assignee_any': 'assignee.any', 'projects_any': 'projects.any', 'sections_any': 'sections.any', 'teams_any': 'teams.any', 'followers_any': 'followers.any', 'created_at_after': 'created_at.after', 'created_at_before': 'created_at.before', 'modified_at_after': 'modified_at.after', 'modified_at_before': 'modified_at.before', 'due_on_after': 'due_on.after', 'due_on_before': 'due_on.before', 'resource_subtype': 'resource_subtype', 'sort_by': 'sort_by', 'sort_ascending': 'sort_ascending'},
        ('projects', 'list'): {'limit': 'limit', 'offset': 'offset', 'workspace': 'workspace', 'team': 'team', 'archived': 'archived'},
        ('projects', 'get'): {'project_gid': 'project_gid'},
        ('task_projects', 'list'): {'task_gid': 'task_gid', 'limit': 'limit', 'offset': 'offset'},
        ('team_projects', 'list'): {'team_gid': 'team_gid', 'limit': 'limit', 'offset': 'offset', 'archived': 'archived'},
        ('workspace_projects', 'list'): {'workspace_gid': 'workspace_gid', 'limit': 'limit', 'offset': 'offset', 'archived': 'archived'},
        ('workspaces', 'list'): {'limit': 'limit', 'offset': 'offset'},
        ('workspaces', 'get'): {'workspace_gid': 'workspace_gid'},
        ('users', 'list'): {'limit': 'limit', 'offset': 'offset', 'workspace': 'workspace', 'team': 'team'},
        ('users', 'get'): {'user_gid': 'user_gid'},
        ('workspace_users', 'list'): {'workspace_gid': 'workspace_gid', 'limit': 'limit', 'offset': 'offset'},
        ('team_users', 'list'): {'team_gid': 'team_gid', 'limit': 'limit', 'offset': 'offset'},
        ('teams', 'get'): {'team_gid': 'team_gid'},
        ('workspace_teams', 'list'): {'workspace_gid': 'workspace_gid', 'limit': 'limit', 'offset': 'offset'},
        ('user_teams', 'list'): {'user_gid': 'user_gid', 'organization': 'organization', 'limit': 'limit', 'offset': 'offset'},
        ('attachments', 'list'): {'parent': 'parent', 'limit': 'limit', 'offset': 'offset'},
        ('attachments', 'get'): {'attachment_gid': 'attachment_gid'},
        ('attachments', 'download'): {'attachment_gid': 'attachment_gid', 'range_header': 'range_header'},
        ('workspace_tags', 'list'): {'workspace_gid': 'workspace_gid', 'limit': 'limit', 'offset': 'offset'},
        ('tags', 'get'): {'tag_gid': 'tag_gid'},
        ('project_sections', 'list'): {'project_gid': 'project_gid', 'limit': 'limit', 'offset': 'offset'},
        ('sections', 'get'): {'section_gid': 'section_gid'},
        ('task_subtasks', 'list'): {'task_gid': 'task_gid', 'limit': 'limit', 'offset': 'offset'},
        ('task_dependencies', 'list'): {'task_gid': 'task_gid', 'limit': 'limit', 'offset': 'offset'},
        ('task_dependents', 'list'): {'task_gid': 'task_gid', 'limit': 'limit', 'offset': 'offset'},
    }

    def __init__(
        self,
        auth_config: AsanaAuthConfig | None = None,
        external_user_id: str | None = None,
        airbyte_client_id: str | None = None,
        airbyte_client_secret: str | None = None,
        on_token_refresh: Any | None = None    ):
        """
        Initialize a new asana connector instance.

        Supports both local and hosted execution modes:
        - Local mode: Provide `auth_config` for direct API calls
        - Hosted mode: Provide `external_user_id`, `airbyte_client_id`, and `airbyte_client_secret` for hosted execution

        Args:
            auth_config: Typed authentication configuration (required for local mode)
            external_user_id: External user ID (required for hosted mode)
            airbyte_client_id: Airbyte OAuth client ID (required for hosted mode)
            airbyte_client_secret: Airbyte OAuth client secret (required for hosted mode)
            on_token_refresh: Optional callback for OAuth2 token refresh persistence.
                Called with new_tokens dict when tokens are refreshed. Can be sync or async.
                Example: lambda tokens: save_to_database(tokens)
        Examples:
            # Local mode (direct API calls)
            connector = AsanaConnector(auth_config=AsanaAuthConfig(access_token="...", refresh_token="...", client_id="...", client_secret="..."))
            # Hosted mode (executed on Airbyte cloud)
            connector = AsanaConnector(
                external_user_id="user-123",
                airbyte_client_id="client_abc123",
                airbyte_client_secret="secret_xyz789"
            )

            # Local mode with OAuth2 token refresh callback
            def save_tokens(new_tokens: dict) -> None:
                # Persist updated tokens to your storage (file, database, etc.)
                with open("tokens.json", "w") as f:
                    json.dump(new_tokens, f)

            connector = AsanaConnector(
                auth_config=AsanaAuthConfig(access_token="...", refresh_token="..."),
                on_token_refresh=save_tokens
            )
        """
        # Hosted mode: external_user_id, airbyte_client_id, and airbyte_client_secret provided
        if external_user_id and airbyte_client_id and airbyte_client_secret:
            from ._vendored.connector_sdk.executor import HostedExecutor
            self._executor = HostedExecutor(
                external_user_id=external_user_id,
                airbyte_client_id=airbyte_client_id,
                airbyte_client_secret=airbyte_client_secret,
                connector_definition_id=str(AsanaConnectorModel.id),
            )
        else:
            # Local mode: auth_config required
            if not auth_config:
                raise ValueError(
                    "Either provide (external_user_id, airbyte_client_id, airbyte_client_secret) for hosted mode "
                    "or auth_config for local mode"
                )

            from ._vendored.connector_sdk.executor import LocalExecutor

            # Build config_values dict from server variables
            config_values = None

            # Multi-auth connector: detect auth scheme from auth_config type
            auth_scheme: str | None = None
            if auth_config:
                if isinstance(auth_config, AsanaOauth2AuthConfig):
                    auth_scheme = "oauth2"
                if isinstance(auth_config, AsanaPersonalAccessTokenAuthConfig):
                    auth_scheme = "personalAccessToken"

            self._executor = LocalExecutor(
                model=AsanaConnectorModel,
                auth_config=auth_config.model_dump() if auth_config else None,
                auth_scheme=auth_scheme,
                config_values=config_values,
                on_token_refresh=on_token_refresh
            )

            # Update base_url with server variables if provided

        # Initialize entity query objects
        self.tasks = TasksQuery(self)
        self.project_tasks = ProjectTasksQuery(self)
        self.workspace_task_search = WorkspaceTaskSearchQuery(self)
        self.projects = ProjectsQuery(self)
        self.task_projects = TaskProjectsQuery(self)
        self.team_projects = TeamProjectsQuery(self)
        self.workspace_projects = WorkspaceProjectsQuery(self)
        self.workspaces = WorkspacesQuery(self)
        self.users = UsersQuery(self)
        self.workspace_users = WorkspaceUsersQuery(self)
        self.team_users = TeamUsersQuery(self)
        self.teams = TeamsQuery(self)
        self.workspace_teams = WorkspaceTeamsQuery(self)
        self.user_teams = UserTeamsQuery(self)
        self.attachments = AttachmentsQuery(self)
        self.workspace_tags = WorkspaceTagsQuery(self)
        self.tags = TagsQuery(self)
        self.project_sections = ProjectSectionsQuery(self)
        self.sections = SectionsQuery(self)
        self.task_subtasks = TaskSubtasksQuery(self)
        self.task_dependencies = TaskDependenciesQuery(self)
        self.task_dependents = TaskDependentsQuery(self)

    # ===== TYPED EXECUTE METHOD (Recommended Interface) =====

    @overload
    async def execute(
        self,
        entity: Literal["tasks"],
        action: Literal["list"],
        params: "TasksListParams"
    ) -> "TasksListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["project_tasks"],
        action: Literal["list"],
        params: "ProjectTasksListParams"
    ) -> "ProjectTasksListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["tasks"],
        action: Literal["get"],
        params: "TasksGetParams"
    ) -> "TasksGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspace_task_search"],
        action: Literal["list"],
        params: "WorkspaceTaskSearchListParams"
    ) -> "WorkspaceTaskSearchListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["projects"],
        action: Literal["list"],
        params: "ProjectsListParams"
    ) -> "ProjectsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["projects"],
        action: Literal["get"],
        params: "ProjectsGetParams"
    ) -> "ProjectsGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["task_projects"],
        action: Literal["list"],
        params: "TaskProjectsListParams"
    ) -> "TaskProjectsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["team_projects"],
        action: Literal["list"],
        params: "TeamProjectsListParams"
    ) -> "TeamProjectsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspace_projects"],
        action: Literal["list"],
        params: "WorkspaceProjectsListParams"
    ) -> "WorkspaceProjectsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspaces"],
        action: Literal["list"],
        params: "WorkspacesListParams"
    ) -> "WorkspacesListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspaces"],
        action: Literal["get"],
        params: "WorkspacesGetParams"
    ) -> "WorkspacesGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["users"],
        action: Literal["list"],
        params: "UsersListParams"
    ) -> "UsersListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["users"],
        action: Literal["get"],
        params: "UsersGetParams"
    ) -> "UsersGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspace_users"],
        action: Literal["list"],
        params: "WorkspaceUsersListParams"
    ) -> "WorkspaceUsersListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["team_users"],
        action: Literal["list"],
        params: "TeamUsersListParams"
    ) -> "TeamUsersListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["teams"],
        action: Literal["get"],
        params: "TeamsGetParams"
    ) -> "TeamsGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspace_teams"],
        action: Literal["list"],
        params: "WorkspaceTeamsListParams"
    ) -> "WorkspaceTeamsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["user_teams"],
        action: Literal["list"],
        params: "UserTeamsListParams"
    ) -> "UserTeamsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["attachments"],
        action: Literal["list"],
        params: "AttachmentsListParams"
    ) -> "AttachmentsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["attachments"],
        action: Literal["get"],
        params: "AttachmentsGetParams"
    ) -> "AttachmentsGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["attachments"],
        action: Literal["download"],
        params: "AttachmentsDownloadParams"
    ) -> "AsyncIterator[bytes]": ...

    @overload
    async def execute(
        self,
        entity: Literal["workspace_tags"],
        action: Literal["list"],
        params: "WorkspaceTagsListParams"
    ) -> "WorkspaceTagsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["tags"],
        action: Literal["get"],
        params: "TagsGetParams"
    ) -> "TagsGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["project_sections"],
        action: Literal["list"],
        params: "ProjectSectionsListParams"
    ) -> "ProjectSectionsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["sections"],
        action: Literal["get"],
        params: "SectionsGetParams"
    ) -> "SectionsGetResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["task_subtasks"],
        action: Literal["list"],
        params: "TaskSubtasksListParams"
    ) -> "TaskSubtasksListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["task_dependencies"],
        action: Literal["list"],
        params: "TaskDependenciesListParams"
    ) -> "TaskDependenciesListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["task_dependents"],
        action: Literal["list"],
        params: "TaskDependentsListParams"
    ) -> "TaskDependentsListResult": ...


    @overload
    async def execute(
        self,
        entity: str,
        action: str,
        params: dict[str, Any]
    ) -> AsanaExecuteResult[Any] | AsanaExecuteResultWithMeta[Any, Any] | Any: ...

    async def execute(
        self,
        entity: str,
        action: str,
        params: dict[str, Any] | None = None
    ) -> Any:
        """
        Execute an entity operation with full type safety.

        This is the recommended interface for blessed connectors as it:
        - Uses the same signature as non-blessed connectors
        - Provides full IDE autocomplete for entity/action/params
        - Makes migration from generic to blessed connectors seamless

        Args:
            entity: Entity name (e.g., "customers")
            action: Operation action (e.g., "create", "get", "list")
            params: Operation parameters (typed based on entity+action)

        Returns:
            Typed response based on the operation

        Example:
            customer = await connector.execute(
                entity="customers",
                action="get",
                params={"id": "cus_123"}
            )
        """
        from ._vendored.connector_sdk.executor import ExecutionConfig

        # Remap parameter names from snake_case (TypedDict keys) to API parameter names
        if params:
            param_map = self._PARAM_MAP.get((entity, action), {})
            if param_map:
                params = {param_map.get(k, k): v for k, v in params.items()}

        # Use ExecutionConfig for both local and hosted executors
        config = ExecutionConfig(
            entity=entity,
            action=action,
            params=params
        )

        result = await self._executor.execute(config)

        if not result.success:
            raise RuntimeError(f"Execution failed: {result.error}")

        # Check if this operation has extractors configured
        has_extractors = self._EXTRACTOR_MAP.get((entity, action), False)

        if has_extractors:
            # With extractors - return Pydantic envelope with data and meta
            if result.meta is not None:
                return AsanaExecuteResultWithMeta[Any, Any](
                    data=result.data,
                    meta=result.meta
                )
            else:
                return AsanaExecuteResult[Any](data=result.data)
        else:
            # No extractors - return raw response data
            return result.data



class TasksQuery:
    """
    Query class for Tasks entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        limit: int | None = None,
        offset: str | None = None,
        project: str | None = None,
        workspace: str | None = None,
        section: str | None = None,
        assignee: str | None = None,
        completed_since: str | None = None,
        modified_since: str | None = None,
        **kwargs
    ) -> TasksListResult:
        """
        Returns a paginated list of tasks

        Args:
            limit: Number of items to return per page
            offset: Pagination offset token
            project: The project to filter tasks on
            workspace: The workspace to filter tasks on
            section: The workspace to filter tasks on
            assignee: The assignee to filter tasks on
            completed_since: Only return tasks that have been completed since this time
            modified_since: Only return tasks that have been completed since this time
            **kwargs: Additional parameters

        Returns:
            TasksListResult
        """
        params = {k: v for k, v in {
            "limit": limit,
            "offset": offset,
            "project": project,
            "workspace": workspace,
            "section": section,
            "assignee": assignee,
            "completed_since": completed_since,
            "modified_since": modified_since,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("tasks", "list", params)
        # Cast generic envelope to concrete typed result
        return TasksListResult(
            data=result.data,
            meta=result.meta        )



    async def get(
        self,
        task_gid: str,
        **kwargs
    ) -> TasksGetResult:
        """
        Get a single task by its ID

        Args:
            task_gid: Task GID
            **kwargs: Additional parameters

        Returns:
            TasksGetResult
        """
        params = {k: v for k, v in {
            "task_gid": task_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("tasks", "get", params)
        # Cast generic envelope to concrete typed result
        return TasksGetResult(
            data=result.data        )



class ProjectTasksQuery:
    """
    Query class for ProjectTasks entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        project_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        completed_since: str | None = None,
        **kwargs
    ) -> ProjectTasksListResult:
        """
        Returns all tasks in a project

        Args:
            project_gid: Project GID to list tasks from
            limit: Number of items to return per page
            offset: Pagination offset token
            completed_since: Only return tasks that have been completed since this time
            **kwargs: Additional parameters

        Returns:
            ProjectTasksListResult
        """
        params = {k: v for k, v in {
            "project_gid": project_gid,
            "limit": limit,
            "offset": offset,
            "completed_since": completed_since,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("project_tasks", "list", params)
        # Cast generic envelope to concrete typed result
        return ProjectTasksListResult(
            data=result.data,
            meta=result.meta        )



class WorkspaceTaskSearchQuery:
    """
    Query class for WorkspaceTaskSearch entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        workspace_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        text: str | None = None,
        completed: bool | None = None,
        assignee_any: str | None = None,
        projects_any: str | None = None,
        sections_any: str | None = None,
        teams_any: str | None = None,
        followers_any: str | None = None,
        created_at_after: str | None = None,
        created_at_before: str | None = None,
        modified_at_after: str | None = None,
        modified_at_before: str | None = None,
        due_on_after: str | None = None,
        due_on_before: str | None = None,
        resource_subtype: str | None = None,
        sort_by: str | None = None,
        sort_ascending: bool | None = None,
        **kwargs
    ) -> WorkspaceTaskSearchListResult:
        """
        Returns tasks that match the specified search criteria. Note - This endpoint requires a premium Asana account.

        Args:
            workspace_gid: Workspace GID to search tasks in
            limit: Number of items to return per page
            offset: Pagination offset token
            text: Search text to filter tasks
            completed: Filter by completion status
            assignee_any: Comma-separated list of assignee GIDs
            projects_any: Comma-separated list of project GIDs
            sections_any: Comma-separated list of section GIDs
            teams_any: Comma-separated list of team GIDs
            followers_any: Comma-separated list of follower GIDs
            created_at_after: Filter tasks created after this date (ISO 8601 format)
            created_at_before: Filter tasks created before this date (ISO 8601 format)
            modified_at_after: Filter tasks modified after this date (ISO 8601 format)
            modified_at_before: Filter tasks modified before this date (ISO 8601 format)
            due_on_after: Filter tasks due after this date (ISO 8601 date format)
            due_on_before: Filter tasks due before this date (ISO 8601 date format)
            resource_subtype: Filter by task resource subtype (e.g., default_task, milestone)
            sort_by: Field to sort by (e.g., created_at, modified_at, due_date)
            sort_ascending: Sort order (true for ascending, false for descending)
            **kwargs: Additional parameters

        Returns:
            WorkspaceTaskSearchListResult
        """
        params = {k: v for k, v in {
            "workspace_gid": workspace_gid,
            "limit": limit,
            "offset": offset,
            "text": text,
            "completed": completed,
            "assignee.any": assignee_any,
            "projects.any": projects_any,
            "sections.any": sections_any,
            "teams.any": teams_any,
            "followers.any": followers_any,
            "created_at.after": created_at_after,
            "created_at.before": created_at_before,
            "modified_at.after": modified_at_after,
            "modified_at.before": modified_at_before,
            "due_on.after": due_on_after,
            "due_on.before": due_on_before,
            "resource_subtype": resource_subtype,
            "sort_by": sort_by,
            "sort_ascending": sort_ascending,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspace_task_search", "list", params)
        # Cast generic envelope to concrete typed result
        return WorkspaceTaskSearchListResult(
            data=result.data,
            meta=result.meta        )



class ProjectsQuery:
    """
    Query class for Projects entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        limit: int | None = None,
        offset: str | None = None,
        workspace: str | None = None,
        team: str | None = None,
        archived: bool | None = None,
        **kwargs
    ) -> ProjectsListResult:
        """
        Returns a paginated list of projects

        Args:
            limit: Number of items to return per page
            offset: Pagination offset token
            workspace: The workspace to filter projects on
            team: The team to filter projects on
            archived: Filter by archived status
            **kwargs: Additional parameters

        Returns:
            ProjectsListResult
        """
        params = {k: v for k, v in {
            "limit": limit,
            "offset": offset,
            "workspace": workspace,
            "team": team,
            "archived": archived,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("projects", "list", params)
        # Cast generic envelope to concrete typed result
        return ProjectsListResult(
            data=result.data,
            meta=result.meta        )



    async def get(
        self,
        project_gid: str,
        **kwargs
    ) -> ProjectsGetResult:
        """
        Get a single project by its ID

        Args:
            project_gid: Project GID
            **kwargs: Additional parameters

        Returns:
            ProjectsGetResult
        """
        params = {k: v for k, v in {
            "project_gid": project_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("projects", "get", params)
        # Cast generic envelope to concrete typed result
        return ProjectsGetResult(
            data=result.data        )



class TaskProjectsQuery:
    """
    Query class for TaskProjects entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        task_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> TaskProjectsListResult:
        """
        Returns all projects a task is in

        Args:
            task_gid: Task GID to list projects from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            TaskProjectsListResult
        """
        params = {k: v for k, v in {
            "task_gid": task_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("task_projects", "list", params)
        # Cast generic envelope to concrete typed result
        return TaskProjectsListResult(
            data=result.data,
            meta=result.meta        )



class TeamProjectsQuery:
    """
    Query class for TeamProjects entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        team_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        archived: bool | None = None,
        **kwargs
    ) -> TeamProjectsListResult:
        """
        Returns all projects for a team

        Args:
            team_gid: Team GID to list projects from
            limit: Number of items to return per page
            offset: Pagination offset token
            archived: Filter by archived status
            **kwargs: Additional parameters

        Returns:
            TeamProjectsListResult
        """
        params = {k: v for k, v in {
            "team_gid": team_gid,
            "limit": limit,
            "offset": offset,
            "archived": archived,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("team_projects", "list", params)
        # Cast generic envelope to concrete typed result
        return TeamProjectsListResult(
            data=result.data,
            meta=result.meta        )



class WorkspaceProjectsQuery:
    """
    Query class for WorkspaceProjects entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        workspace_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        archived: bool | None = None,
        **kwargs
    ) -> WorkspaceProjectsListResult:
        """
        Returns all projects in a workspace

        Args:
            workspace_gid: Workspace GID to list projects from
            limit: Number of items to return per page
            offset: Pagination offset token
            archived: Filter by archived status
            **kwargs: Additional parameters

        Returns:
            WorkspaceProjectsListResult
        """
        params = {k: v for k, v in {
            "workspace_gid": workspace_gid,
            "limit": limit,
            "offset": offset,
            "archived": archived,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspace_projects", "list", params)
        # Cast generic envelope to concrete typed result
        return WorkspaceProjectsListResult(
            data=result.data,
            meta=result.meta        )



class WorkspacesQuery:
    """
    Query class for Workspaces entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> WorkspacesListResult:
        """
        Returns a paginated list of workspaces

        Args:
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            WorkspacesListResult
        """
        params = {k: v for k, v in {
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspaces", "list", params)
        # Cast generic envelope to concrete typed result
        return WorkspacesListResult(
            data=result.data,
            meta=result.meta        )



    async def get(
        self,
        workspace_gid: str,
        **kwargs
    ) -> WorkspacesGetResult:
        """
        Get a single workspace by its ID

        Args:
            workspace_gid: Workspace GID
            **kwargs: Additional parameters

        Returns:
            WorkspacesGetResult
        """
        params = {k: v for k, v in {
            "workspace_gid": workspace_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspaces", "get", params)
        # Cast generic envelope to concrete typed result
        return WorkspacesGetResult(
            data=result.data        )



class UsersQuery:
    """
    Query class for Users entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        limit: int | None = None,
        offset: str | None = None,
        workspace: str | None = None,
        team: str | None = None,
        **kwargs
    ) -> UsersListResult:
        """
        Returns a paginated list of users

        Args:
            limit: Number of items to return per page
            offset: Pagination offset token
            workspace: The workspace to filter users on
            team: The team to filter users on
            **kwargs: Additional parameters

        Returns:
            UsersListResult
        """
        params = {k: v for k, v in {
            "limit": limit,
            "offset": offset,
            "workspace": workspace,
            "team": team,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("users", "list", params)
        # Cast generic envelope to concrete typed result
        return UsersListResult(
            data=result.data,
            meta=result.meta        )



    async def get(
        self,
        user_gid: str,
        **kwargs
    ) -> UsersGetResult:
        """
        Get a single user by their ID

        Args:
            user_gid: User GID
            **kwargs: Additional parameters

        Returns:
            UsersGetResult
        """
        params = {k: v for k, v in {
            "user_gid": user_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("users", "get", params)
        # Cast generic envelope to concrete typed result
        return UsersGetResult(
            data=result.data        )



class WorkspaceUsersQuery:
    """
    Query class for WorkspaceUsers entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        workspace_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> WorkspaceUsersListResult:
        """
        Returns all users in a workspace

        Args:
            workspace_gid: Workspace GID to list users from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            WorkspaceUsersListResult
        """
        params = {k: v for k, v in {
            "workspace_gid": workspace_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspace_users", "list", params)
        # Cast generic envelope to concrete typed result
        return WorkspaceUsersListResult(
            data=result.data,
            meta=result.meta        )



class TeamUsersQuery:
    """
    Query class for TeamUsers entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        team_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> TeamUsersListResult:
        """
        Returns all users in a team

        Args:
            team_gid: Team GID to list users from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            TeamUsersListResult
        """
        params = {k: v for k, v in {
            "team_gid": team_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("team_users", "list", params)
        # Cast generic envelope to concrete typed result
        return TeamUsersListResult(
            data=result.data,
            meta=result.meta        )



class TeamsQuery:
    """
    Query class for Teams entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def get(
        self,
        team_gid: str,
        **kwargs
    ) -> TeamsGetResult:
        """
        Get a single team by its ID

        Args:
            team_gid: Team GID
            **kwargs: Additional parameters

        Returns:
            TeamsGetResult
        """
        params = {k: v for k, v in {
            "team_gid": team_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("teams", "get", params)
        # Cast generic envelope to concrete typed result
        return TeamsGetResult(
            data=result.data        )



class WorkspaceTeamsQuery:
    """
    Query class for WorkspaceTeams entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        workspace_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> WorkspaceTeamsListResult:
        """
        Returns all teams in a workspace

        Args:
            workspace_gid: Workspace GID to list teams from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            WorkspaceTeamsListResult
        """
        params = {k: v for k, v in {
            "workspace_gid": workspace_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspace_teams", "list", params)
        # Cast generic envelope to concrete typed result
        return WorkspaceTeamsListResult(
            data=result.data,
            meta=result.meta        )



class UserTeamsQuery:
    """
    Query class for UserTeams entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        user_gid: str,
        organization: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> UserTeamsListResult:
        """
        Returns all teams a user is a member of

        Args:
            user_gid: User GID to list teams from
            organization: The workspace or organization to filter teams on
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            UserTeamsListResult
        """
        params = {k: v for k, v in {
            "user_gid": user_gid,
            "organization": organization,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("user_teams", "list", params)
        # Cast generic envelope to concrete typed result
        return UserTeamsListResult(
            data=result.data,
            meta=result.meta        )



class AttachmentsQuery:
    """
    Query class for Attachments entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        parent: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> AttachmentsListResult:
        """
        Returns a list of attachments for an object (task, project, etc.)

        Args:
            parent: Globally unique identifier for the object to fetch attachments for (e.g., a task GID)
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            AttachmentsListResult
        """
        params = {k: v for k, v in {
            "parent": parent,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("attachments", "list", params)
        # Cast generic envelope to concrete typed result
        return AttachmentsListResult(
            data=result.data,
            meta=result.meta        )



    async def get(
        self,
        attachment_gid: str,
        **kwargs
    ) -> AttachmentsGetResult:
        """
        Get details for a single attachment by its GID

        Args:
            attachment_gid: Globally unique identifier for the attachment
            **kwargs: Additional parameters

        Returns:
            AttachmentsGetResult
        """
        params = {k: v for k, v in {
            "attachment_gid": attachment_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("attachments", "get", params)
        # Cast generic envelope to concrete typed result
        return AttachmentsGetResult(
            data=result.data        )



    async def download(
        self,
        attachment_gid: str,
        range_header: str | None = None,
        **kwargs
    ) -> AsyncIterator[bytes]:
        """
        Downloads the file content of an attachment. This operation first retrieves the attachment
metadata to get the download_url, then downloads the file from that URL.


        Args:
            attachment_gid: Globally unique identifier for the attachment
            range_header: Optional Range header for partial downloads (e.g., 'bytes=0-99')
            **kwargs: Additional parameters

        Returns:
            AsyncIterator[bytes]
        """
        params = {k: v for k, v in {
            "attachment_gid": attachment_gid,
            "range_header": range_header,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("attachments", "download", params)
        return result


    async def download_local(
        self,
        attachment_gid: str,
        path: str,
        range_header: str | None = None,
        **kwargs
    ) -> Path:
        """
        Downloads the file content of an attachment. This operation first retrieves the attachment
metadata to get the download_url, then downloads the file from that URL.
 and save to file.

        Args:
            attachment_gid: Globally unique identifier for the attachment
            range_header: Optional Range header for partial downloads (e.g., 'bytes=0-99')
            path: File path to save downloaded content
            **kwargs: Additional parameters

        Returns:
            str: Path to the downloaded file
        """
        from ._vendored.connector_sdk import save_download

        # Get the async iterator
        content_iterator = await self.download(
            attachment_gid=attachment_gid,
            range_header=range_header,
            **kwargs
        )

        return await save_download(content_iterator, path)


class WorkspaceTagsQuery:
    """
    Query class for WorkspaceTags entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        workspace_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> WorkspaceTagsListResult:
        """
        Returns all tags in a workspace

        Args:
            workspace_gid: Workspace GID to list tags from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            WorkspaceTagsListResult
        """
        params = {k: v for k, v in {
            "workspace_gid": workspace_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("workspace_tags", "list", params)
        # Cast generic envelope to concrete typed result
        return WorkspaceTagsListResult(
            data=result.data,
            meta=result.meta        )



class TagsQuery:
    """
    Query class for Tags entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def get(
        self,
        tag_gid: str,
        **kwargs
    ) -> TagsGetResult:
        """
        Get a single tag by its ID

        Args:
            tag_gid: Tag GID
            **kwargs: Additional parameters

        Returns:
            TagsGetResult
        """
        params = {k: v for k, v in {
            "tag_gid": tag_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("tags", "get", params)
        # Cast generic envelope to concrete typed result
        return TagsGetResult(
            data=result.data        )



class ProjectSectionsQuery:
    """
    Query class for ProjectSections entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        project_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> ProjectSectionsListResult:
        """
        Returns all sections in a project

        Args:
            project_gid: Project GID to list sections from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            ProjectSectionsListResult
        """
        params = {k: v for k, v in {
            "project_gid": project_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("project_sections", "list", params)
        # Cast generic envelope to concrete typed result
        return ProjectSectionsListResult(
            data=result.data,
            meta=result.meta        )



class SectionsQuery:
    """
    Query class for Sections entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def get(
        self,
        section_gid: str,
        **kwargs
    ) -> SectionsGetResult:
        """
        Get a single section by its ID

        Args:
            section_gid: Section GID
            **kwargs: Additional parameters

        Returns:
            SectionsGetResult
        """
        params = {k: v for k, v in {
            "section_gid": section_gid,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("sections", "get", params)
        # Cast generic envelope to concrete typed result
        return SectionsGetResult(
            data=result.data        )



class TaskSubtasksQuery:
    """
    Query class for TaskSubtasks entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        task_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> TaskSubtasksListResult:
        """
        Returns all subtasks of a task

        Args:
            task_gid: Task GID to list subtasks from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            TaskSubtasksListResult
        """
        params = {k: v for k, v in {
            "task_gid": task_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("task_subtasks", "list", params)
        # Cast generic envelope to concrete typed result
        return TaskSubtasksListResult(
            data=result.data,
            meta=result.meta        )



class TaskDependenciesQuery:
    """
    Query class for TaskDependencies entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        task_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> TaskDependenciesListResult:
        """
        Returns all tasks that this task depends on

        Args:
            task_gid: Task GID to list dependencies from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            TaskDependenciesListResult
        """
        params = {k: v for k, v in {
            "task_gid": task_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("task_dependencies", "list", params)
        # Cast generic envelope to concrete typed result
        return TaskDependenciesListResult(
            data=result.data,
            meta=result.meta        )



class TaskDependentsQuery:
    """
    Query class for TaskDependents entity operations.
    """

    def __init__(self, connector: AsanaConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        task_gid: str,
        limit: int | None = None,
        offset: str | None = None,
        **kwargs
    ) -> TaskDependentsListResult:
        """
        Returns all tasks that depend on this task

        Args:
            task_gid: Task GID to list dependents from
            limit: Number of items to return per page
            offset: Pagination offset token
            **kwargs: Additional parameters

        Returns:
            TaskDependentsListResult
        """
        params = {k: v for k, v in {
            "task_gid": task_gid,
            "limit": limit,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("task_dependents", "list", params)
        # Cast generic envelope to concrete typed result
        return TaskDependentsListResult(
            data=result.data,
            meta=result.meta        )





"""
Connector model for asana.

This file is auto-generated from the connector definition at build time.
DO NOT EDIT MANUALLY - changes will be overwritten on next generation.
"""

from __future__ import annotations

from ._vendored.connector_sdk.types import (
    Action,
    AuthConfig,
    AuthOption,
    AuthType,
    ConnectorModel,
    EndpointDefinition,
    EntityDefinition,
)
from ._vendored.connector_sdk.schema.security import (
    AirbyteAuthConfig,
    AuthConfigFieldSpec,
)
from ._vendored.connector_sdk.schema.components import (
    PathOverrideConfig,
)
from uuid import (
    UUID,
)

AsanaConnectorModel: ConnectorModel = ConnectorModel(
    id=UUID('d0243522-dccf-4978-8ba0-37ed47a0bdbf'),
    name='asana',
    version='0.1.5',
    base_url='https://app.asana.com/api/1.0',
    auth=AuthConfig(
        options=[
            AuthOption(
                scheme_name='oauth2',
                type=AuthType.OAUTH2,
                config={
                    'header': 'Authorization',
                    'prefix': 'Bearer',
                    'refresh_url': 'https://app.asana.com/-/oauth_token',
                },
                user_config_spec=AirbyteAuthConfig(
                    title='OAuth 2',
                    type='object',
                    required=['refresh_token', 'client_id', 'client_secret'],
                    properties={
                        'access_token': AuthConfigFieldSpec(
                            title='Access Token',
                            description='OAuth access token for API requests',
                            airbyte_secret=True,
                        ),
                        'refresh_token': AuthConfigFieldSpec(
                            title='Refresh Token',
                            description='OAuth refresh token for automatic token renewal',
                            airbyte_secret=True,
                        ),
                        'client_id': AuthConfigFieldSpec(
                            title='Client ID',
                            description='Connected App Consumer Key',
                        ),
                        'client_secret': AuthConfigFieldSpec(
                            title='Client Secret',
                            description='Connected App Consumer Secret',
                            airbyte_secret=True,
                        ),
                    },
                    auth_mapping={
                        'access_token': '${access_token}',
                        'refresh_token': '${refresh_token}',
                        'client_id': '${client_id}',
                        'client_secret': '${client_secret}',
                    },
                ),
            ),
            AuthOption(
                scheme_name='personalAccessToken',
                type=AuthType.BEARER,
                config={'header': 'Authorization', 'prefix': 'Bearer'},
                user_config_spec=AirbyteAuthConfig(
                    title='Personal Access Token',
                    type='object',
                    required=['token'],
                    properties={
                        'token': AuthConfigFieldSpec(
                            title='Personal Access Token',
                            description='Your Asana Personal Access Token. Generate one at https://app.asana.com/0/my-apps',
                            airbyte_secret=True,
                        ),
                    },
                    auth_mapping={'token': '${token}'},
                ),
            ),
        ],
    ),
    entities=[
        EntityDefinition(
            name='tasks',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/tasks',
                    action=Action.LIST,
                    description='Returns a paginated list of tasks',
                    query_params=[
                        'limit',
                        'offset',
                        'project',
                        'workspace',
                        'section',
                        'assignee',
                        'completed_since',
                        'modified_since',
                    ],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'project': {'type': 'string', 'required': False},
                        'workspace': {'type': 'string', 'required': False},
                        'section': {'type': 'string', 'required': False},
                        'assignee': {'type': 'string', 'required': False},
                        'completed_since': {'type': 'string', 'required': False},
                        'modified_since': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tasks containing compact task objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact task object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                                        'name': {'type': 'string', 'description': 'Task name'},
                                        'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                                        'created_by': {
                                            'type': 'object',
                                            'description': 'User who created the task',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'x-airbyte-entity-name': 'tasks',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/tasks/{task_gid}',
                    action=Action.GET,
                    description='Get a single task by its ID',
                    path_params=['task_gid'],
                    path_params_schema={
                        'task_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Task response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full task object',
                                'properties': {
                                    'gid': {
                                        'type': 'string',
                                        'actual_time_minutes': {
                                            'type': ['integer', 'null'],
                                        },
                                        'assignee': {
                                            'type': 'object',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'name': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                        'assignee_status': {'type': 'string'},
                                        'assignee_section': {
                                            'type': 'object',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'name': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                        'completed': {'type': 'boolean'},
                                        'completed_at': {
                                            'type': ['string', 'null'],
                                        },
                                        'created_at': {'type': 'string'},
                                        'due_at': {
                                            'type': ['string', 'null'],
                                        },
                                        'due_on': {
                                            'type': ['string', 'null'],
                                        },
                                        'followers': {
                                            'type': 'array',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'gid': {'type': 'string'},
                                                    'name': {'type': 'string'},
                                                    'resource_type': {'type': 'string'},
                                                },
                                            },
                                        },
                                        'hearted': {'type': 'boolean'},
                                        'hearts': {'type': 'array'},
                                        'liked': {'type': 'boolean'},
                                        'likes': {'type': 'array'},
                                        'memberships': {
                                            'type': 'array',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'project': {
                                                        'type': 'object',
                                                        'properties': {
                                                            'gid': {'type': 'string'},
                                                            'name': {'type': 'string'},
                                                            'resource_type': {'type': 'string'},
                                                        },
                                                    },
                                                    'section': {
                                                        'type': 'object',
                                                        'properties': {
                                                            'gid': {'type': 'string'},
                                                            'name': {'type': 'string'},
                                                            'resource_type': {'type': 'string'},
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        'modified_at': {'type': 'string'},
                                        'name': {'type': 'string'},
                                        'notes': {'type': 'string'},
                                        'num_hearts': {'type': 'integer'},
                                        'num_likes': {'type': 'integer'},
                                        'parent': {
                                            'type': ['object', 'null'],
                                        },
                                        'permalink_url': {'type': 'string'},
                                        'projects': {
                                            'type': 'array',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'gid': {'type': 'string'},
                                                    'name': {'type': 'string'},
                                                    'resource_type': {'type': 'string'},
                                                },
                                            },
                                        },
                                        'resource_type': {'type': 'string'},
                                        'start_at': {
                                            'type': ['string', 'null'],
                                        },
                                        'start_on': {
                                            'type': ['string', 'null'],
                                        },
                                        'tags': {'type': 'array'},
                                        'resource_subtype': {'type': 'string'},
                                        'workspace': {
                                            'type': 'object',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'name': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact task object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                    'name': {'type': 'string', 'description': 'Task name'},
                    'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                    'created_by': {
                        'type': 'object',
                        'description': 'User who created the task',
                        'properties': {
                            'gid': {'type': 'string'},
                            'resource_type': {'type': 'string'},
                        },
                    },
                },
                'x-airbyte-entity-name': 'tasks',
            },
        ),
        EntityDefinition(
            name='project_tasks',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/projects/{project_gid}/tasks',
                    action=Action.LIST,
                    description='Returns all tasks in a project',
                    query_params=['limit', 'offset', 'completed_since'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'completed_since': {'type': 'string', 'required': False},
                    },
                    path_params=['project_gid'],
                    path_params_schema={
                        'project_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tasks containing compact task objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact task object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                                        'name': {'type': 'string', 'description': 'Task name'},
                                        'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                                        'created_by': {
                                            'type': 'object',
                                            'description': 'User who created the task',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'x-airbyte-entity-name': 'tasks',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='workspace_task_search',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/workspaces/{workspace_gid}/tasks/search',
                    action=Action.LIST,
                    description='Returns tasks that match the specified search criteria. Note - This endpoint requires a premium Asana account.',
                    query_params=[
                        'limit',
                        'offset',
                        'text',
                        'completed',
                        'assignee.any',
                        'projects.any',
                        'sections.any',
                        'teams.any',
                        'followers.any',
                        'created_at.after',
                        'created_at.before',
                        'modified_at.after',
                        'modified_at.before',
                        'due_on.after',
                        'due_on.before',
                        'resource_subtype',
                        'sort_by',
                        'sort_ascending',
                    ],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'text': {'type': 'string', 'required': False},
                        'completed': {'type': 'boolean', 'required': False},
                        'assignee.any': {'type': 'string', 'required': False},
                        'projects.any': {'type': 'string', 'required': False},
                        'sections.any': {'type': 'string', 'required': False},
                        'teams.any': {'type': 'string', 'required': False},
                        'followers.any': {'type': 'string', 'required': False},
                        'created_at.after': {'type': 'string', 'required': False},
                        'created_at.before': {'type': 'string', 'required': False},
                        'modified_at.after': {'type': 'string', 'required': False},
                        'modified_at.before': {'type': 'string', 'required': False},
                        'due_on.after': {'type': 'string', 'required': False},
                        'due_on.before': {'type': 'string', 'required': False},
                        'resource_subtype': {'type': 'string', 'required': False},
                        'sort_by': {'type': 'string', 'required': False},
                        'sort_ascending': {'type': 'boolean', 'required': False},
                    },
                    path_params=['workspace_gid'],
                    path_params_schema={
                        'workspace_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tasks containing compact task objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact task object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                                        'name': {'type': 'string', 'description': 'Task name'},
                                        'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                                        'created_by': {
                                            'type': 'object',
                                            'description': 'User who created the task',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'x-airbyte-entity-name': 'tasks',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='projects',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/projects',
                    action=Action.LIST,
                    description='Returns a paginated list of projects',
                    query_params=[
                        'limit',
                        'offset',
                        'workspace',
                        'team',
                        'archived',
                    ],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'workspace': {'type': 'string', 'required': False},
                        'team': {'type': 'string', 'required': False},
                        'archived': {'type': 'boolean', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of projects containing compact project objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact project object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (project)'},
                                        'name': {'type': 'string', 'description': 'Project name'},
                                    },
                                    'x-airbyte-entity-name': 'projects',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/projects/{project_gid}',
                    action=Action.GET,
                    description='Get a single project by its ID',
                    path_params=['project_gid'],
                    path_params_schema={
                        'project_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Project response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full project object',
                                'properties': {
                                    'gid': {'type': 'string'},
                                    'archived': {'type': 'boolean'},
                                    'color': {
                                        'type': ['string', 'null'],
                                    },
                                    'completed': {'type': 'boolean'},
                                    'completed_at': {
                                        'type': ['string', 'null'],
                                    },
                                    'created_at': {'type': 'string'},
                                    'current_status': {
                                        'type': ['object', 'null'],
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'author': {
                                                'type': 'object',
                                                'properties': {
                                                    'gid': {'type': 'string'},
                                                    'name': {'type': 'string'},
                                                    'resource_type': {'type': 'string'},
                                                },
                                            },
                                            'color': {'type': 'string'},
                                            'created_at': {'type': 'string'},
                                            'created_by': {
                                                'type': 'object',
                                                'properties': {
                                                    'gid': {'type': 'string'},
                                                    'name': {'type': 'string'},
                                                    'resource_type': {'type': 'string'},
                                                },
                                            },
                                            'modified_at': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                            'text': {'type': 'string'},
                                            'title': {'type': 'string'},
                                        },
                                    },
                                    'current_status_update': {
                                        'type': ['object', 'null'],
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                            'resource_subtype': {'type': 'string'},
                                            'title': {'type': 'string'},
                                        },
                                    },
                                    'custom_fields': {'type': 'array'},
                                    'default_access_level': {'type': 'string'},
                                    'default_view': {'type': 'string'},
                                    'due_on': {
                                        'type': ['string', 'null'],
                                    },
                                    'due_date': {
                                        'type': ['string', 'null'],
                                    },
                                    'followers': {
                                        'type': 'array',
                                        'items': {
                                            'type': 'object',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'name': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'members': {
                                        'type': 'array',
                                        'items': {
                                            'type': 'object',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'name': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'minimum_access_level_for_customization': {'type': 'string'},
                                    'minimum_access_level_for_sharing': {'type': 'string'},
                                    'modified_at': {'type': 'string'},
                                    'name': {'type': 'string'},
                                    'notes': {'type': 'string'},
                                    'owner': {
                                        'type': 'object',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                        },
                                    },
                                    'permalink_url': {'type': 'string'},
                                    'privacy_setting': {'type': 'string'},
                                    'public': {'type': 'boolean'},
                                    'resource_type': {'type': 'string'},
                                    'start_on': {
                                        'type': ['string', 'null'],
                                    },
                                    'team': {
                                        'type': 'object',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                        },
                                    },
                                    'workspace': {
                                        'type': 'object',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                        },
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact project object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (project)'},
                    'name': {'type': 'string', 'description': 'Project name'},
                },
                'x-airbyte-entity-name': 'projects',
            },
        ),
        EntityDefinition(
            name='task_projects',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/tasks/{task_gid}/projects',
                    action=Action.LIST,
                    description='Returns all projects a task is in',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['task_gid'],
                    path_params_schema={
                        'task_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of projects containing compact project objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact project object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (project)'},
                                        'name': {'type': 'string', 'description': 'Project name'},
                                    },
                                    'x-airbyte-entity-name': 'projects',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='team_projects',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/teams/{team_gid}/projects',
                    action=Action.LIST,
                    description='Returns all projects for a team',
                    query_params=['limit', 'offset', 'archived'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'archived': {'type': 'boolean', 'required': False},
                    },
                    path_params=['team_gid'],
                    path_params_schema={
                        'team_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of projects containing compact project objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact project object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (project)'},
                                        'name': {'type': 'string', 'description': 'Project name'},
                                    },
                                    'x-airbyte-entity-name': 'projects',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='workspace_projects',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/workspaces/{workspace_gid}/projects',
                    action=Action.LIST,
                    description='Returns all projects in a workspace',
                    query_params=['limit', 'offset', 'archived'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'archived': {'type': 'boolean', 'required': False},
                    },
                    path_params=['workspace_gid'],
                    path_params_schema={
                        'workspace_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of projects containing compact project objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact project object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (project)'},
                                        'name': {'type': 'string', 'description': 'Project name'},
                                    },
                                    'x-airbyte-entity-name': 'projects',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='workspaces',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/workspaces',
                    action=Action.LIST,
                    description='Returns a paginated list of workspaces',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of workspaces containing compact workspace objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact workspace object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (workspace)'},
                                        'name': {'type': 'string', 'description': 'Workspace name'},
                                    },
                                    'x-airbyte-entity-name': 'workspaces',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/workspaces/{workspace_gid}',
                    action=Action.GET,
                    description='Get a single workspace by its ID',
                    path_params=['workspace_gid'],
                    path_params_schema={
                        'workspace_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Workspace response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full workspace object',
                                'properties': {
                                    'gid': {'type': 'string'},
                                    'resource_type': {'type': 'string'},
                                    'name': {'type': 'string'},
                                    'email_domains': {
                                        'type': 'array',
                                        'items': {'type': 'string'},
                                    },
                                    'is_organization': {'type': 'boolean'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact workspace object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (workspace)'},
                    'name': {'type': 'string', 'description': 'Workspace name'},
                },
                'x-airbyte-entity-name': 'workspaces',
            },
        ),
        EntityDefinition(
            name='users',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/users',
                    action=Action.LIST,
                    description='Returns a paginated list of users',
                    query_params=[
                        'limit',
                        'offset',
                        'workspace',
                        'team',
                    ],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                        'workspace': {'type': 'string', 'required': False},
                        'team': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of users containing compact user objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact user object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (user)'},
                                        'name': {'type': 'string', 'description': 'User name'},
                                    },
                                    'x-airbyte-entity-name': 'users',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/users/{user_gid}',
                    action=Action.GET,
                    description='Get a single user by their ID',
                    path_params=['user_gid'],
                    path_params_schema={
                        'user_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'User response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full user object',
                                'properties': {
                                    'gid': {'type': 'string'},
                                    'email': {'type': 'string'},
                                    'name': {'type': 'string'},
                                    'photo': {
                                        'type': ['object', 'null'],
                                    },
                                    'resource_type': {'type': 'string'},
                                    'workspaces': {
                                        'type': 'array',
                                        'items': {
                                            'type': 'object',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'name': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact user object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (user)'},
                    'name': {'type': 'string', 'description': 'User name'},
                },
                'x-airbyte-entity-name': 'users',
            },
        ),
        EntityDefinition(
            name='workspace_users',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/workspaces/{workspace_gid}/users',
                    action=Action.LIST,
                    description='Returns all users in a workspace',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['workspace_gid'],
                    path_params_schema={
                        'workspace_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of users containing compact user objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact user object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (user)'},
                                        'name': {'type': 'string', 'description': 'User name'},
                                    },
                                    'x-airbyte-entity-name': 'users',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='team_users',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/teams/{team_gid}/users',
                    action=Action.LIST,
                    description='Returns all users in a team',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['team_gid'],
                    path_params_schema={
                        'team_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of users containing compact user objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact user object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (user)'},
                                        'name': {'type': 'string', 'description': 'User name'},
                                    },
                                    'x-airbyte-entity-name': 'users',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='teams',
            actions=[Action.GET],
            endpoints={
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/teams/{team_gid}',
                    action=Action.GET,
                    description='Get a single team by its ID',
                    path_params=['team_gid'],
                    path_params_schema={
                        'team_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Team response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full team object',
                                'properties': {
                                    'gid': {'type': 'string'},
                                    'name': {'type': 'string'},
                                    'organization': {
                                        'type': 'object',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                        },
                                    },
                                    'permalink_url': {'type': 'string'},
                                    'resource_type': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact team object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (team)'},
                    'name': {'type': 'string', 'description': 'Team name'},
                },
                'x-airbyte-entity-name': 'teams',
            },
        ),
        EntityDefinition(
            name='workspace_teams',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/workspaces/{workspace_gid}/teams',
                    action=Action.LIST,
                    description='Returns all teams in a workspace',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['workspace_gid'],
                    path_params_schema={
                        'workspace_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of teams containing compact team objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact team object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (team)'},
                                        'name': {'type': 'string', 'description': 'Team name'},
                                    },
                                    'x-airbyte-entity-name': 'teams',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='user_teams',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/users/{user_gid}/teams',
                    action=Action.LIST,
                    description='Returns all teams a user is a member of',
                    query_params=['organization', 'limit', 'offset'],
                    query_params_schema={
                        'organization': {'type': 'string', 'required': True},
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['user_gid'],
                    path_params_schema={
                        'user_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of teams containing compact team objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact team object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (team)'},
                                        'name': {'type': 'string', 'description': 'Team name'},
                                    },
                                    'x-airbyte-entity-name': 'teams',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='attachments',
            actions=[Action.LIST, Action.GET, Action.DOWNLOAD],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/attachments',
                    action=Action.LIST,
                    description='Returns a list of attachments for an object (task, project, etc.)',
                    query_params=['parent', 'limit', 'offset'],
                    query_params_schema={
                        'parent': {'type': 'string', 'required': True},
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of attachments containing compact attachment objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact attachment object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (attachment)'},
                                        'name': {'type': 'string', 'description': 'The name of the attachment'},
                                        'resource_subtype': {'type': 'string', 'description': 'The type of the attachment (e.g., external, dropbox, gdrive, asana, etc.)'},
                                    },
                                    'x-airbyte-entity-name': 'attachments',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/attachments/{attachment_gid}',
                    action=Action.GET,
                    description='Get details for a single attachment by its GID',
                    path_params=['attachment_gid'],
                    path_params_schema={
                        'attachment_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Attachment response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full attachment object',
                                'properties': {
                                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                    'resource_type': {'type': 'string', 'description': 'Resource type (attachment)'},
                                    'name': {'type': 'string', 'description': 'The name of the attachment'},
                                    'resource_subtype': {'type': 'string', 'description': 'The type of the attachment (e.g., external, dropbox, gdrive, etc.)'},
                                    'created_at': {
                                        'type': 'string',
                                        'format': 'date-time',
                                        'description': 'The time at which this attachment was created',
                                    },
                                    'download_url': {
                                        'type': ['string', 'null'],
                                        'format': 'uri',
                                        'description': 'The URL where the attachment can be downloaded. May be null if the attachment is hosted externally.',
                                    },
                                    'permanent_url': {
                                        'type': ['string', 'null'],
                                        'format': 'uri',
                                        'description': 'The permanent URL of the attachment. May be null if the attachment does not support permanent URLs.',
                                    },
                                    'host': {'type': 'string', 'description': 'The service hosting the attachment (asana, dropbox, gdrive, box, etc.)'},
                                    'parent': {
                                        'type': 'object',
                                        'description': 'The parent object this attachment is attached to',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_subtype': {'type': 'string', 'description': 'The subtype of the parent resource'},
                                        },
                                    },
                                    'view_url': {
                                        'type': ['string', 'null'],
                                        'format': 'uri',
                                        'description': 'The URL where the attachment can be viewed in a browser',
                                    },
                                    'size': {
                                        'type': ['integer', 'null'],
                                        'description': 'The size of the attachment in bytes',
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
                Action.DOWNLOAD: EndpointDefinition(
                    method='GET',
                    path='/attachments/{attachment_gid}:download',
                    path_override=PathOverrideConfig(
                        path='/attachments/{attachment_gid}',
                    ),
                    action=Action.DOWNLOAD,
                    description='Downloads the file content of an attachment. This operation first retrieves the attachment\nmetadata to get the download_url, then downloads the file from that URL.\n',
                    path_params=['attachment_gid'],
                    path_params_schema={
                        'attachment_gid': {'type': 'string', 'required': True},
                    },
                    file_field='data.download_url',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact attachment object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (attachment)'},
                    'name': {'type': 'string', 'description': 'The name of the attachment'},
                    'resource_subtype': {'type': 'string', 'description': 'The type of the attachment (e.g., external, dropbox, gdrive, asana, etc.)'},
                },
                'x-airbyte-entity-name': 'attachments',
            },
        ),
        EntityDefinition(
            name='workspace_tags',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/workspaces/{workspace_gid}/tags',
                    action=Action.LIST,
                    description='Returns all tags in a workspace',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['workspace_gid'],
                    path_params_schema={
                        'workspace_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tags containing compact tag objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact tag object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (tag)'},
                                        'name': {'type': 'string', 'description': 'Tag name'},
                                    },
                                    'x-airbyte-entity-name': 'tags',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='tags',
            actions=[Action.GET],
            endpoints={
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/tags/{tag_gid}',
                    action=Action.GET,
                    description='Get a single tag by its ID',
                    path_params=['tag_gid'],
                    path_params_schema={
                        'tag_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Tag response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full tag object',
                                'properties': {
                                    'gid': {'type': 'string'},
                                    'resource_type': {'type': 'string'},
                                    'name': {'type': 'string'},
                                    'color': {'type': 'string'},
                                    'created_at': {'type': 'string'},
                                    'followers': {'type': 'array'},
                                    'notes': {'type': 'string'},
                                    'permalink_url': {'type': 'string'},
                                    'workspace': {
                                        'type': 'object',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                        },
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact tag object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (tag)'},
                    'name': {'type': 'string', 'description': 'Tag name'},
                },
                'x-airbyte-entity-name': 'tags',
            },
        ),
        EntityDefinition(
            name='project_sections',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/projects/{project_gid}/sections',
                    action=Action.LIST,
                    description='Returns all sections in a project',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['project_gid'],
                    path_params_schema={
                        'project_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of sections containing compact section objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact section object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (section)'},
                                        'name': {'type': 'string', 'description': 'Section name'},
                                    },
                                    'x-airbyte-entity-name': 'sections',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='sections',
            actions=[Action.GET],
            endpoints={
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/sections/{section_gid}',
                    action=Action.GET,
                    description='Get a single section by its ID',
                    path_params=['section_gid'],
                    path_params_schema={
                        'section_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Section response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Full section object',
                                'properties': {
                                    'gid': {'type': 'string'},
                                    'resource_type': {'type': 'string'},
                                    'name': {'type': 'string'},
                                    'created_at': {'type': 'string'},
                                    'project': {
                                        'type': 'object',
                                        'properties': {
                                            'gid': {'type': 'string'},
                                            'name': {'type': 'string'},
                                            'resource_type': {'type': 'string'},
                                        },
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Compact section object',
                'properties': {
                    'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                    'resource_type': {'type': 'string', 'description': 'Resource type (section)'},
                    'name': {'type': 'string', 'description': 'Section name'},
                },
                'x-airbyte-entity-name': 'sections',
            },
        ),
        EntityDefinition(
            name='task_subtasks',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/tasks/{task_gid}/subtasks',
                    action=Action.LIST,
                    description='Returns all subtasks of a task',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['task_gid'],
                    path_params_schema={
                        'task_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tasks containing compact task objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact task object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                                        'name': {'type': 'string', 'description': 'Task name'},
                                        'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                                        'created_by': {
                                            'type': 'object',
                                            'description': 'User who created the task',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'x-airbyte-entity-name': 'tasks',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='task_dependencies',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/tasks/{task_gid}/dependencies',
                    action=Action.LIST,
                    description='Returns all tasks that this task depends on',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['task_gid'],
                    path_params_schema={
                        'task_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tasks containing compact task objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact task object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                                        'name': {'type': 'string', 'description': 'Task name'},
                                        'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                                        'created_by': {
                                            'type': 'object',
                                            'description': 'User who created the task',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'x-airbyte-entity-name': 'tasks',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
        EntityDefinition(
            name='task_dependents',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/tasks/{task_gid}/dependents',
                    action=Action.LIST,
                    description='Returns all tasks that depend on this task',
                    query_params=['limit', 'offset'],
                    query_params_schema={
                        'limit': {
                            'type': 'integer',
                            'required': False,
                            'default': 20,
                        },
                        'offset': {'type': 'string', 'required': False},
                    },
                    path_params=['task_gid'],
                    path_params_schema={
                        'task_gid': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Paginated list of tasks containing compact task objects',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'Compact task object',
                                    'properties': {
                                        'gid': {'type': 'string', 'description': 'Globally unique identifier'},
                                        'resource_type': {'type': 'string', 'description': 'Resource type (task)'},
                                        'name': {'type': 'string', 'description': 'Task name'},
                                        'resource_subtype': {'type': 'string', 'description': 'Task subtype'},
                                        'created_by': {
                                            'type': 'object',
                                            'description': 'User who created the task',
                                            'properties': {
                                                'gid': {'type': 'string'},
                                                'resource_type': {'type': 'string'},
                                            },
                                        },
                                    },
                                    'x-airbyte-entity-name': 'tasks',
                                },
                            },
                            'next_page': {
                                'type': ['object', 'null'],
                                'properties': {
                                    'offset': {'type': 'string'},
                                    'path': {'type': 'string'},
                                    'uri': {'type': 'string'},
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    meta_extractor={'next_page': '$.next_page'},
                ),
            },
        ),
    ],
)


"""
Pydantic models for asana connector.

This module contains Pydantic models used for authentication configuration
and response envelope types.
"""

from __future__ import annotations

from pydantic import BaseModel, ConfigDict, Field
from typing import TypeVar, Generic, Union, Any
from typing import Optional

# Authentication configuration - multiple options available

class AsanaOauth2AuthConfig(BaseModel):
    """OAuth 2"""

    model_config = ConfigDict(extra="forbid")

    access_token: Optional[str] = None
    """OAuth access token for API requests"""
    refresh_token: str
    """OAuth refresh token for automatic token renewal"""
    client_id: str
    """Connected App Consumer Key"""
    client_secret: str
    """Connected App Consumer Secret"""

class AsanaPersonalAccessTokenAuthConfig(BaseModel):
    """Personal Access Token"""

    model_config = ConfigDict(extra="forbid")

    token: str
    """Your Asana Personal Access Token. Generate one at https://app.asana.com/0/my-apps"""

AsanaAuthConfig = AsanaOauth2AuthConfig | AsanaPersonalAccessTokenAuthConfig

# ===== RESPONSE TYPE DEFINITIONS (PYDANTIC) =====

class TaskCompactCreatedBy(BaseModel):
    """User who created the task"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class TaskCompact(BaseModel):
    """Compact task object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_subtype: Union[str, Any] = Field(default=None)
    created_by: Union[TaskCompactCreatedBy, Any] = Field(default=None)

class Task(BaseModel):
    """Full task object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)

class TaskResponse(BaseModel):
    """Task response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Task, Any] = Field(default=None)

class TasksListNextPage(BaseModel):
    """Nested schema for TasksList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class TasksList(BaseModel):
    """Paginated list of tasks containing compact task objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[TaskCompact], Any] = Field(default=None)
    next_page: Union[TasksListNextPage | None, Any] = Field(default=None)

class ProjectCompact(BaseModel):
    """Compact project object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)

class ProjectWorkspace(BaseModel):
    """Nested schema for Project.workspace"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectOwner(BaseModel):
    """Nested schema for Project.owner"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectCurrentStatusCreatedBy(BaseModel):
    """Nested schema for ProjectCurrentStatus.created_by"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectCurrentStatusAuthor(BaseModel):
    """Nested schema for ProjectCurrentStatus.author"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectCurrentStatus(BaseModel):
    """Nested schema for Project.current_status"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    author: Union[ProjectCurrentStatusAuthor, Any] = Field(default=None)
    color: Union[str, Any] = Field(default=None)
    created_at: Union[str, Any] = Field(default=None)
    created_by: Union[ProjectCurrentStatusCreatedBy, Any] = Field(default=None)
    modified_at: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    text: Union[str, Any] = Field(default=None)
    title: Union[str, Any] = Field(default=None)

class ProjectMembersItem(BaseModel):
    """Nested schema for Project.members_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectTeam(BaseModel):
    """Nested schema for Project.team"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectFollowersItem(BaseModel):
    """Nested schema for Project.followers_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class ProjectCurrentStatusUpdate(BaseModel):
    """Nested schema for Project.current_status_update"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    resource_subtype: Union[str, Any] = Field(default=None)
    title: Union[str, Any] = Field(default=None)

class Project(BaseModel):
    """Full project object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    archived: Union[bool, Any] = Field(default=None)
    color: Union[str | None, Any] = Field(default=None)
    completed: Union[bool, Any] = Field(default=None)
    completed_at: Union[str | None, Any] = Field(default=None)
    created_at: Union[str, Any] = Field(default=None)
    current_status: Union[ProjectCurrentStatus | None, Any] = Field(default=None)
    current_status_update: Union[ProjectCurrentStatusUpdate | None, Any] = Field(default=None)
    custom_fields: Union[list[Any], Any] = Field(default=None)
    default_access_level: Union[str, Any] = Field(default=None)
    default_view: Union[str, Any] = Field(default=None)
    due_on: Union[str | None, Any] = Field(default=None)
    due_date: Union[str | None, Any] = Field(default=None)
    followers: Union[list[ProjectFollowersItem], Any] = Field(default=None)
    members: Union[list[ProjectMembersItem], Any] = Field(default=None)
    minimum_access_level_for_customization: Union[str, Any] = Field(default=None)
    minimum_access_level_for_sharing: Union[str, Any] = Field(default=None)
    modified_at: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    notes: Union[str, Any] = Field(default=None)
    owner: Union[ProjectOwner, Any] = Field(default=None)
    permalink_url: Union[str, Any] = Field(default=None)
    privacy_setting: Union[str, Any] = Field(default=None)
    public: Union[bool, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    start_on: Union[str | None, Any] = Field(default=None)
    team: Union[ProjectTeam, Any] = Field(default=None)
    workspace: Union[ProjectWorkspace, Any] = Field(default=None)

class ProjectResponse(BaseModel):
    """Project response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Project, Any] = Field(default=None)

class ProjectsListNextPage(BaseModel):
    """Nested schema for ProjectsList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class ProjectsList(BaseModel):
    """Paginated list of projects containing compact project objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[ProjectCompact], Any] = Field(default=None)
    next_page: Union[ProjectsListNextPage | None, Any] = Field(default=None)

class WorkspaceCompact(BaseModel):
    """Compact workspace object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)

class Workspace(BaseModel):
    """Full workspace object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    email_domains: Union[list[str], Any] = Field(default=None)
    is_organization: Union[bool, Any] = Field(default=None)

class WorkspaceResponse(BaseModel):
    """Workspace response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Workspace, Any] = Field(default=None)

class WorkspacesListNextPage(BaseModel):
    """Nested schema for WorkspacesList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class WorkspacesList(BaseModel):
    """Paginated list of workspaces containing compact workspace objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[WorkspaceCompact], Any] = Field(default=None)
    next_page: Union[WorkspacesListNextPage | None, Any] = Field(default=None)

class UserCompact(BaseModel):
    """Compact user object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)

class UserWorkspacesItem(BaseModel):
    """Nested schema for User.workspaces_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class User(BaseModel):
    """Full user object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    email: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    photo: Union[dict[str, Any] | None, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    workspaces: Union[list[UserWorkspacesItem], Any] = Field(default=None)

class UserResponse(BaseModel):
    """User response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[User, Any] = Field(default=None)

class UsersListNextPage(BaseModel):
    """Nested schema for UsersList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class UsersList(BaseModel):
    """Paginated list of users containing compact user objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[UserCompact], Any] = Field(default=None)
    next_page: Union[UsersListNextPage | None, Any] = Field(default=None)

class TeamCompact(BaseModel):
    """Compact team object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)

class TeamOrganization(BaseModel):
    """Nested schema for Team.organization"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class Team(BaseModel):
    """Full team object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    organization: Union[TeamOrganization, Any] = Field(default=None)
    permalink_url: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class TeamResponse(BaseModel):
    """Team response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Team, Any] = Field(default=None)

class TeamsListNextPage(BaseModel):
    """Nested schema for TeamsList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class TeamsList(BaseModel):
    """Paginated list of teams containing compact team objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[TeamCompact], Any] = Field(default=None)
    next_page: Union[TeamsListNextPage | None, Any] = Field(default=None)

class AttachmentCompact(BaseModel):
    """Compact attachment object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_subtype: Union[str, Any] = Field(default=None)

class AttachmentParent(BaseModel):
    """The parent object this attachment is attached to"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_subtype: Union[str, Any] = Field(default=None, description="The subtype of the parent resource")
    """The subtype of the parent resource"""

class Attachment(BaseModel):
    """Full attachment object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_subtype: Union[str, Any] = Field(default=None)
    created_at: Union[str, Any] = Field(default=None)
    download_url: Union[str | None, Any] = Field(default=None)
    permanent_url: Union[str | None, Any] = Field(default=None)
    host: Union[str, Any] = Field(default=None)
    parent: Union[AttachmentParent, Any] = Field(default=None)
    view_url: Union[str | None, Any] = Field(default=None)
    size: Union[int | None, Any] = Field(default=None)

class AttachmentResponse(BaseModel):
    """Attachment response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Attachment, Any] = Field(default=None)

class AttachmentsListNextPage(BaseModel):
    """Nested schema for AttachmentsList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class AttachmentsList(BaseModel):
    """Paginated list of attachments containing compact attachment objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[AttachmentCompact], Any] = Field(default=None)
    next_page: Union[AttachmentsListNextPage | None, Any] = Field(default=None)

class TagCompact(BaseModel):
    """Compact tag object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)

class TagWorkspace(BaseModel):
    """Nested schema for Tag.workspace"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class Tag(BaseModel):
    """Full tag object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    color: Union[str, Any] = Field(default=None)
    created_at: Union[str, Any] = Field(default=None)
    followers: Union[list[Any], Any] = Field(default=None)
    notes: Union[str, Any] = Field(default=None)
    permalink_url: Union[str, Any] = Field(default=None)
    workspace: Union[TagWorkspace, Any] = Field(default=None)

class TagResponse(BaseModel):
    """Tag response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Tag, Any] = Field(default=None)

class TagsListNextPage(BaseModel):
    """Nested schema for TagsList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class TagsList(BaseModel):
    """Paginated list of tags containing compact tag objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[TagCompact], Any] = Field(default=None)
    next_page: Union[TagsListNextPage | None, Any] = Field(default=None)

class SectionCompact(BaseModel):
    """Compact section object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)

class SectionProject(BaseModel):
    """Nested schema for Section.project"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)

class Section(BaseModel):
    """Full section object"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    gid: Union[str, Any] = Field(default=None)
    resource_type: Union[str, Any] = Field(default=None)
    name: Union[str, Any] = Field(default=None)
    created_at: Union[str, Any] = Field(default=None)
    project: Union[SectionProject, Any] = Field(default=None)

class SectionResponse(BaseModel):
    """Section response wrapper"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[Section, Any] = Field(default=None)

class SectionsListNextPage(BaseModel):
    """Nested schema for SectionsList.next_page"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    offset: Union[str, Any] = Field(default=None)
    path: Union[str, Any] = Field(default=None)
    uri: Union[str, Any] = Field(default=None)

class SectionsList(BaseModel):
    """Paginated list of sections containing compact section objects"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    data: Union[list[SectionCompact], Any] = Field(default=None)
    next_page: Union[SectionsListNextPage | None, Any] = Field(default=None)

# ===== METADATA TYPE DEFINITIONS (PYDANTIC) =====
# Meta types for operations that extract metadata (e.g., pagination info)

class TasksListResultMeta(BaseModel):
    """Metadata for tasks.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class ProjectTasksListResultMeta(BaseModel):
    """Metadata for project_tasks.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class WorkspaceTaskSearchListResultMeta(BaseModel):
    """Metadata for workspace_task_search.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class ProjectsListResultMeta(BaseModel):
    """Metadata for projects.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class TaskProjectsListResultMeta(BaseModel):
    """Metadata for task_projects.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class TeamProjectsListResultMeta(BaseModel):
    """Metadata for team_projects.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class WorkspaceProjectsListResultMeta(BaseModel):
    """Metadata for workspace_projects.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class WorkspacesListResultMeta(BaseModel):
    """Metadata for workspaces.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class UsersListResultMeta(BaseModel):
    """Metadata for users.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class WorkspaceUsersListResultMeta(BaseModel):
    """Metadata for workspace_users.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class TeamUsersListResultMeta(BaseModel):
    """Metadata for team_users.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class WorkspaceTeamsListResultMeta(BaseModel):
    """Metadata for workspace_teams.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class UserTeamsListResultMeta(BaseModel):
    """Metadata for user_teams.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class AttachmentsListResultMeta(BaseModel):
    """Metadata for attachments.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class WorkspaceTagsListResultMeta(BaseModel):
    """Metadata for workspace_tags.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class ProjectSectionsListResultMeta(BaseModel):
    """Metadata for project_sections.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class TaskSubtasksListResultMeta(BaseModel):
    """Metadata for task_subtasks.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class TaskDependenciesListResultMeta(BaseModel):
    """Metadata for task_dependencies.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

class TaskDependentsListResultMeta(BaseModel):
    """Metadata for task_dependents.list operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    next_page: Union[dict[str, Any] | None, Any] = Field(default=None)

# ===== RESPONSE ENVELOPE MODELS =====

# Type variables for generic envelope models
T = TypeVar('T')
S = TypeVar('S')


class AsanaExecuteResult(BaseModel, Generic[T]):
    """Response envelope with data only.

    Used for actions that return data without metadata.
    """
    model_config = ConfigDict(extra="forbid")

    data: T
    """Response data containing the result of the action."""


class AsanaExecuteResultWithMeta(AsanaExecuteResult[T], Generic[T, S]):
    """Response envelope with data and metadata.

    Used for actions that return both data and metadata (e.g., pagination info).
    """
    meta: S
    """Metadata about the response (e.g., pagination cursors, record counts)."""


# ===== OPERATION RESULT TYPE ALIASES =====

# Concrete type aliases for each operation result.
# These provide simpler, more readable type annotations than using the generic forms.

TasksListResult = AsanaExecuteResultWithMeta[list[TaskCompact], TasksListResultMeta]
"""Result type for tasks.list operation with data and metadata."""

ProjectTasksListResult = AsanaExecuteResultWithMeta[list[TaskCompact], ProjectTasksListResultMeta]
"""Result type for project_tasks.list operation with data and metadata."""

TasksGetResult = AsanaExecuteResult[Task]
"""Result type for tasks.get operation."""

WorkspaceTaskSearchListResult = AsanaExecuteResultWithMeta[list[TaskCompact], WorkspaceTaskSearchListResultMeta]
"""Result type for workspace_task_search.list operation with data and metadata."""

ProjectsListResult = AsanaExecuteResultWithMeta[list[ProjectCompact], ProjectsListResultMeta]
"""Result type for projects.list operation with data and metadata."""

ProjectsGetResult = AsanaExecuteResult[Project]
"""Result type for projects.get operation."""

TaskProjectsListResult = AsanaExecuteResultWithMeta[list[ProjectCompact], TaskProjectsListResultMeta]
"""Result type for task_projects.list operation with data and metadata."""

TeamProjectsListResult = AsanaExecuteResultWithMeta[list[ProjectCompact], TeamProjectsListResultMeta]
"""Result type for team_projects.list operation with data and metadata."""

WorkspaceProjectsListResult = AsanaExecuteResultWithMeta[list[ProjectCompact], WorkspaceProjectsListResultMeta]
"""Result type for workspace_projects.list operation with data and metadata."""

WorkspacesListResult = AsanaExecuteResultWithMeta[list[WorkspaceCompact], WorkspacesListResultMeta]
"""Result type for workspaces.list operation with data and metadata."""

WorkspacesGetResult = AsanaExecuteResult[Workspace]
"""Result type for workspaces.get operation."""

UsersListResult = AsanaExecuteResultWithMeta[list[UserCompact], UsersListResultMeta]
"""Result type for users.list operation with data and metadata."""

UsersGetResult = AsanaExecuteResult[User]
"""Result type for users.get operation."""

WorkspaceUsersListResult = AsanaExecuteResultWithMeta[list[UserCompact], WorkspaceUsersListResultMeta]
"""Result type for workspace_users.list operation with data and metadata."""

TeamUsersListResult = AsanaExecuteResultWithMeta[list[UserCompact], TeamUsersListResultMeta]
"""Result type for team_users.list operation with data and metadata."""

TeamsGetResult = AsanaExecuteResult[Team]
"""Result type for teams.get operation."""

WorkspaceTeamsListResult = AsanaExecuteResultWithMeta[list[TeamCompact], WorkspaceTeamsListResultMeta]
"""Result type for workspace_teams.list operation with data and metadata."""

UserTeamsListResult = AsanaExecuteResultWithMeta[list[TeamCompact], UserTeamsListResultMeta]
"""Result type for user_teams.list operation with data and metadata."""

AttachmentsListResult = AsanaExecuteResultWithMeta[list[AttachmentCompact], AttachmentsListResultMeta]
"""Result type for attachments.list operation with data and metadata."""

AttachmentsGetResult = AsanaExecuteResult[Attachment]
"""Result type for attachments.get operation."""

WorkspaceTagsListResult = AsanaExecuteResultWithMeta[list[TagCompact], WorkspaceTagsListResultMeta]
"""Result type for workspace_tags.list operation with data and metadata."""

TagsGetResult = AsanaExecuteResult[Tag]
"""Result type for tags.get operation."""

ProjectSectionsListResult = AsanaExecuteResultWithMeta[list[SectionCompact], ProjectSectionsListResultMeta]
"""Result type for project_sections.list operation with data and metadata."""

SectionsGetResult = AsanaExecuteResult[Section]
"""Result type for sections.get operation."""

TaskSubtasksListResult = AsanaExecuteResultWithMeta[list[TaskCompact], TaskSubtasksListResultMeta]
"""Result type for task_subtasks.list operation with data and metadata."""

TaskDependenciesListResult = AsanaExecuteResultWithMeta[list[TaskCompact], TaskDependenciesListResultMeta]
"""Result type for task_dependencies.list operation with data and metadata."""

TaskDependentsListResult = AsanaExecuteResultWithMeta[list[TaskCompact], TaskDependentsListResultMeta]
"""Result type for task_dependents.list operation with data and metadata."""




"""
Type definitions for asana connector.
"""
from __future__ import annotations

# Use typing_extensions.TypedDict for Pydantic compatibility on Python < 3.12
try:
    from typing_extensions import TypedDict, NotRequired
except ImportError:
    from typing import TypedDict, NotRequired  # type: ignore[attr-defined]



# ===== NESTED PARAM TYPE DEFINITIONS =====
# Nested parameter schemas discovered during parameter extraction

# ===== OPERATION PARAMS TYPE DEFINITIONS =====

class TasksListParams(TypedDict):
    """Parameters for tasks.list operation"""
    limit: NotRequired[int]
    offset: NotRequired[str]
    project: NotRequired[str]
    workspace: NotRequired[str]
    section: NotRequired[str]
    assignee: NotRequired[str]
    completed_since: NotRequired[str]
    modified_since: NotRequired[str]

class ProjectTasksListParams(TypedDict):
    """Parameters for project_tasks.list operation"""
    project_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]
    completed_since: NotRequired[str]

class TasksGetParams(TypedDict):
    """Parameters for tasks.get operation"""
    task_gid: str

class WorkspaceTaskSearchListParams(TypedDict):
    """Parameters for workspace_task_search.list operation"""
    workspace_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]
    text: NotRequired[str]
    completed: NotRequired[bool]
    assignee_any: NotRequired[str]
    projects_any: NotRequired[str]
    sections_any: NotRequired[str]
    teams_any: NotRequired[str]
    followers_any: NotRequired[str]
    created_at_after: NotRequired[str]
    created_at_before: NotRequired[str]
    modified_at_after: NotRequired[str]
    modified_at_before: NotRequired[str]
    due_on_after: NotRequired[str]
    due_on_before: NotRequired[str]
    resource_subtype: NotRequired[str]
    sort_by: NotRequired[str]
    sort_ascending: NotRequired[bool]

class ProjectsListParams(TypedDict):
    """Parameters for projects.list operation"""
    limit: NotRequired[int]
    offset: NotRequired[str]
    workspace: NotRequired[str]
    team: NotRequired[str]
    archived: NotRequired[bool]

class ProjectsGetParams(TypedDict):
    """Parameters for projects.get operation"""
    project_gid: str

class TaskProjectsListParams(TypedDict):
    """Parameters for task_projects.list operation"""
    task_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class TeamProjectsListParams(TypedDict):
    """Parameters for team_projects.list operation"""
    team_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]
    archived: NotRequired[bool]

class WorkspaceProjectsListParams(TypedDict):
    """Parameters for workspace_projects.list operation"""
    workspace_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]
    archived: NotRequired[bool]

class WorkspacesListParams(TypedDict):
    """Parameters for workspaces.list operation"""
    limit: NotRequired[int]
    offset: NotRequired[str]

class WorkspacesGetParams(TypedDict):
    """Parameters for workspaces.get operation"""
    workspace_gid: str

class UsersListParams(TypedDict):
    """Parameters for users.list operation"""
    limit: NotRequired[int]
    offset: NotRequired[str]
    workspace: NotRequired[str]
    team: NotRequired[str]

class UsersGetParams(TypedDict):
    """Parameters for users.get operation"""
    user_gid: str

class WorkspaceUsersListParams(TypedDict):
    """Parameters for workspace_users.list operation"""
    workspace_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class TeamUsersListParams(TypedDict):
    """Parameters for team_users.list operation"""
    team_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class TeamsGetParams(TypedDict):
    """Parameters for teams.get operation"""
    team_gid: str

class WorkspaceTeamsListParams(TypedDict):
    """Parameters for workspace_teams.list operation"""
    workspace_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class UserTeamsListParams(TypedDict):
    """Parameters for user_teams.list operation"""
    user_gid: str
    organization: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class AttachmentsListParams(TypedDict):
    """Parameters for attachments.list operation"""
    parent: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class AttachmentsGetParams(TypedDict):
    """Parameters for attachments.get operation"""
    attachment_gid: str

class AttachmentsDownloadParams(TypedDict):
    """Parameters for attachments.download operation"""
    attachment_gid: str
    range_header: NotRequired[str]

class WorkspaceTagsListParams(TypedDict):
    """Parameters for workspace_tags.list operation"""
    workspace_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class TagsGetParams(TypedDict):
    """Parameters for tags.get operation"""
    tag_gid: str

class ProjectSectionsListParams(TypedDict):
    """Parameters for project_sections.list operation"""
    project_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class SectionsGetParams(TypedDict):
    """Parameters for sections.get operation"""
    section_gid: str

class TaskSubtasksListParams(TypedDict):
    """Parameters for task_subtasks.list operation"""
    task_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class TaskDependenciesListParams(TypedDict):
    """Parameters for task_dependencies.list operation"""
    task_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]

class TaskDependentsListParams(TypedDict):
    """Parameters for task_dependents.list operation"""
    task_gid: str
    limit: NotRequired[int]
    offset: NotRequired[str]
